@@ -13,12 +13,10 @@ class Badge(WebComponent, ClassMixin, AppearanceMixin):
         label (str): The badge label (text showing inside a badge).
 
     Example:
-        Badge('Some Badge')
-
-    Demo:
         from bootwrap import Badge
-        output = Badge('Some Badge')
+        output = Badge('Some Badge').as_dark()
     """
+
     def __init__(self, label):
         super().__init__()
         self.__label = label
@@ -26,7 +24,7 @@ class Badge(WebComponent, ClassMixin, AppearanceMixin):
     def __str__(self):
         classes = 'badge'
         if self._category:
-            classes += f' badge-{self._category}'
+            classes += f' text-bg-{self._category}'
 
         if self.classes:
             classes += f' {self.classes}'
