@@ -12,8 +12,11 @@ from bootwrap import (
     TextInput,
     NumericInput,
     SelectInput,
+    JsonInput,
     HiddenInput,
-    FileInput
+    FileInput,
+    InputGroup,
+    Text,
 )
 from .helper import HelperHTMLParser
 
@@ -22,13 +25,12 @@ from .helper import HelperHTMLParser
 def test_form():
     class Dummy(WebComponent):
         def __str__(self):
-            return '<dummy>dummy</dummy>'
+            return "<dummy>dummy</dummy>"
 
-    form = Form(Dummy()).\
-        on_submit('somelink').\
-        add_classes('someclass')
+    form = Form(Dummy()).on_submit("somelink").add_classes("someclass")
     actual = HelperHTMLParser.parse(str(form))
-    expected = HelperHTMLParser.parse(f'''
+    expected = HelperHTMLParser.parse(
+        f"""
         <form id="{form.identifier}"
             action="somelink"
             class="someclass"
@@ -36,7 +38,8 @@ def test_form():
             enctype="multipart/form-data">
             <dummy>dummy</dummy>
         </form>
-    ''')
+    """
+    )
     assert actual == expected
 
 
@@ -47,28 +50,33 @@ def test_generic_input():
             return f'<xyz id="{self.identifier}">{self._name}</xyz>'
 
     # the label on the same row as input...
-    generic = GenericInput('somelabel', 'somename').\
-        add_classes('someclass')
+    generic = GenericInput("somelabel", "somename").add_classes("someclass")
     actual = HelperHTMLParser.parse(str(generic))
-    expected = HelperHTMLParser.parse(f'''
-        <div class="form-group someclass row">
-            <label class="col-sm-2 col-form-label d-flex align-items-center"
-                for="{generic.identifier}">
-                somelabel
-            </label>
-            <div class="col-sm-10 d-flex align-items-center">
-                <xyz id="{generic.identifier}">somename</xyz>
+    expected = HelperHTMLParser.parse(
+        f"""
+            <div class="someclass form-group row">
+                <label class="col-sm-4 col-form-label"
+                    for="{generic.identifier}">
+                    somelabel
+                </label>
+                <div class="col-sm-8">
+                    <xyz id="{generic.identifier}">somename</xyz>
+                </div>
             </div>
-        </div>
-    ''')
+        """
+    )
+
     assert actual == expected
 
     # the label on the top, input at the bottom...
-    generic = GenericInput('somelabel', 'somename').\
-        add_classes('someclass').\
-        label_on_top()
+    generic = (
+        GenericInput("somelabel", "somename")
+        .add_classes("someclass")
+        .label_on_top()
+    )
     actual = HelperHTMLParser.parse(str(generic))
-    expected = HelperHTMLParser.parse(f'''
+    expected = HelperHTMLParser.parse(
+        f"""
         <div class="form-group someclass">
             <label for="{generic.identifier}">
                 somelabel
@@ -77,77 +85,253 @@ def test_generic_input():
                 <xyz id="{generic.identifier}">somename</xyz>
             </div>
         </div>
-    ''')
+    """
+    )
     assert actual == expected
 
     # with no label...
-    generic = GenericInput(None, 'somename').\
-        add_classes('someclass')
+    generic = GenericInput(None, "somename").add_classes("someclass")
     actual = HelperHTMLParser.parse(str(generic))
-    expected = HelperHTMLParser.parse(f'''
-        <div class="form-group someclass">
+    expected = HelperHTMLParser.parse(
+        f"""
+        <xyz id="{generic.identifier}">somename</xyz>
+    """
+    )
+    assert actual == expected
+
+    # with no label but with tip...
+    generic = (
+        GenericInput(None, "somename")
+        .add_classes("someclass")
+        .with_tip("Here is some tip")
+    )
+    actual = HelperHTMLParser.parse(str(generic))
+    expected = HelperHTMLParser.parse(
+        f"""
             <xyz id="{generic.identifier}">somename</xyz>
-        </div>
-    ''')
+            <small class="form-text text-secondary" style="font-size: 0.75em;">
+                Here is some tip
+            </small>
+    """
+    )
     assert actual == expected
 
 
 @pytest.mark.form
 def test_checkbox_input():
-    checkbox = CheckboxInput('somelabel', 'somename')
+    checkbox = CheckboxInput("somelabel", "somename")
     actual = HelperHTMLParser.parse(str(checkbox))
-    expected = HelperHTMLParser.parse(f'''
-        <div class="form-group row">
-            <label class="col-sm-2 col-form-label d-flex align-items-center"
-                for="{checkbox.identifier}">
-                somelabel
-            </label>
-            <div class="col-sm-10 d-flex align-items-center">
-                <input id="{checkbox.identifier}"
-                    name="somename"
-                    type="checkbox"
-                    class="form-check-input"
-                    autocomplete="off"/>
-            </div>
-        </div>
-    ''')
-    assert actual == expected
-
-    checkbox = CheckboxInput('somelabel', 'somename', True).\
-        as_disabled()
-    actual = HelperHTMLParser.parse(str(checkbox))
-    expected = HelperHTMLParser.parse(f'''
-        <div class="form-group row">
-            <label class="col-sm-2 col-form-label d-flex align-items-center"
-                for="{checkbox.identifier}">
-                somelabel
-            </label>
-            <div class="col-sm-10 d-flex align-items-center">
+    expected = HelperHTMLParser.parse(
+        f"""
+        <div class="form-check">
             <input id="{checkbox.identifier}"
                 name="somename"
-                type="checkbox"
                 class="form-check-input"
-                autocomplete="off" checked disabled/>
+                type="checkbox"
+                value="true"
+                autocomplete="off">
+            </input>
+            <label class="form-check-label" 
+                for="{checkbox.identifier}">
+                somelabel
+                <input type="hidden" name="somename" value="false">
+                </input>
+            </label>
+        </div>
+    """
+    )
+    assert actual == expected
+
+    checkbox = CheckboxInput("somelabel", "somename").label_on_left()
+    actual = HelperHTMLParser.parse(str(checkbox))
+    expected = HelperHTMLParser.parse(
+        f"""
+        <div class="form-group row">
+            <label class="col-sm-4 col-form-label d-flex align-items-center"
+                for="...">
+                somelabel
+            </label>
+            <div class="col-sm-8 d-flex align-items-center">
+                <input id="..."
+                    name="somename"
+                    class="form-check-input"
+                    value="true"
+                    type="checkbox"
+                    autocomplete="off">
+                </input>    
+                <input type="hidden" name="somename" value="false">
+                </input> 
             </div>
         </div>
-    ''')
+    """
+    )
     assert actual == expected
 
+    checkbox = CheckboxInput("somelabel", "somename", True).as_disabled()
+    actual = HelperHTMLParser.parse(str(checkbox))
+    expected = HelperHTMLParser.parse(
+        f"""
+        <div class="form-check">
+            <input id="{checkbox.identifier}"
+                name="somename"
+                class="form-check-input"
+                type="checkbox"
+                value="true"
+                autocomplete="off"
+                checked disabled>
+            </input>
+            <label class="form-check-label" 
+                for="{checkbox.identifier}">
+                somelabel
+                <input type="hidden" name="somename" value="false">
+                </input>
+            </label>
+        </div>
+    """
+    )
+    assert actual == expected
+
+    # testing checkbox as switch
+    checkbox = CheckboxInput("somelabel", "somename", False).as_switch()
+    actual = HelperHTMLParser.parse(str(checkbox))
+    expected = HelperHTMLParser.parse(
+        f"""
+        <div class="form-check form-switch">
+            <input id="{checkbox.identifier}"
+                name="somename"
+                class="form-check-input"
+                type="checkbox"
+                value="true"
+                autocomplete="off">
+            </input>
+            <label class="form-check-label" 
+                for="{checkbox.identifier}">
+                somelabel
+                <input type="hidden" name="somename" value="false">
+                </input>
+            </label>
+        </div>
+    """
+    )
+    assert actual == expected
+
+    # testing checkbox as radio
+    checkbox = CheckboxInput("somelabel", "somename", False).as_radio(1)
+    actual = HelperHTMLParser.parse(str(checkbox))
+    expected = HelperHTMLParser.parse(
+        f"""
+        <div class="form-check">
+            <input id="{checkbox.identifier}"
+                name="somename"
+                class="form-check-input"
+                value="1"
+                type="radio"
+                autocomplete="off">
+            </input>
+            <label class="form-check-label" 
+                for="{checkbox.identifier}">
+                somelabel
+                <input type="hidden" name="somename" value="false">
+                </input>
+            </label>
+        </div>
+    """
+    )
+    assert actual == expected
+
+    # testing checkbox as button
+    checkbox = (
+        CheckboxInput("somelabel", "somename", False).as_button().as_primary()
+    )
+    actual = HelperHTMLParser.parse(str(checkbox))
+    expected = HelperHTMLParser.parse(
+        f"""
+        <input id="{checkbox.identifier}"
+            name="somename"
+            class="btn-check"
+            type="checkbox"
+            value="true"
+            autocomplete="off">
+        </input>
+        <label class="btn btn-primary" 
+            for="{checkbox.identifier}">
+            somelabel
+            <input type="hidden" name="somename" value="false">
+            </input>
+        </label>
+    """
+    )
+    assert actual == expected
+
+    checkbox = (
+        CheckboxInput("somelabel", "somename", False)
+        .as_button()
+        .as_primary()
+        .as_outline()
+    )
+    actual = HelperHTMLParser.parse(str(checkbox))
+    expected = HelperHTMLParser.parse(
+        f"""
+        <input id="{checkbox.identifier}"
+            name="somename"
+            class="btn-check"
+            type="checkbox"
+            value="true"
+            autocomplete="off">
+        </input>
+        <label class="btn btn-outline-primary" 
+            for="{checkbox.identifier}">
+            somelabel
+            <input type="hidden" name="somename" value="false">
+            </input>
+        </label>
+    """
+    )
+    assert actual == expected
+
+    # testing checkbox inline
+    checkbox = CheckboxInput("somelabel", "somename", False).inline()
+    actual = HelperHTMLParser.parse(str(checkbox))
+    expected = HelperHTMLParser.parse(
+        f"""
+        <div class="form-check form-check-inline">
+            <input id="{checkbox.identifier}"
+                name="somename"
+                class="form-check-input"
+                type="checkbox"
+                value="true"
+                autocomplete="off">
+            </input>
+            <label class="form-check-label" 
+                for="{checkbox.identifier}">
+                somelabel
+                <input type="hidden" name="somename" value="false">
+                </input>
+            </label>
+        </div>
+    """
+    )
+    assert actual == expected
+
+    # make sure that reciver is empty
+    assert CheckboxInput("somelabel", "somename", False)._receiver() == ""
+
 
 @pytest.mark.form
 def test_text_input():
     # testing text-input...
     text = TextInput(
-        'somelabel', 'somename', 'somevalue', placeholder='someplaceholder'
+        "somelabel", "somename", "somevalue", placeholder="someplaceholder"
     )
     actual = HelperHTMLParser.parse(str(text))
-    expected = HelperHTMLParser.parse(f'''
+    expected = HelperHTMLParser.parse(
+        f"""
         <div class="form-group row">
-            <label class="col-sm-2 col-form-label d-flex align-items-center"
+            <label class="col-sm-4 col-form-label"
                 for="{text.identifier}">
                 somelabel
             </label>
-            <div class="col-sm-10 d-flex align-items-center">
+            <div class="col-sm-8">
                 <input id="{text.identifier}"
                     name="somename"
                     value="somevalue"
@@ -156,71 +340,77 @@ def test_text_input():
                     placeholder="someplaceholder"/>
             </div>
         </div>
-    ''')
+    """
+    )
     assert actual == expected
 
     # testing email-input...
-    email = TextInput('somelabel', 'somename').for_email()
+    email = TextInput("somelabel", "somename").for_email()
     actual = HelperHTMLParser.parse(str(email))
-    expected = HelperHTMLParser.parse(f'''
+    expected = HelperHTMLParser.parse(
+        f"""
         <div class="form-group row">
-            <label class="col-sm-2 col-form-label d-flex align-items-center"
+            <label class="col-sm-4 col-form-label"
                 for="{email.identifier}">
                 somelabel
             </label>
-            <div class="col-sm-10 d-flex align-items-center">
+            <div class="col-sm-8">
                 <input id="{email.identifier}"
                     name="somename"
                     type="email"
                     class="form-control"/>
             </div>
         </div>
-    ''')
+    """
+    )
     assert actual == expected
 
     # testing password-input...
-    password = TextInput('somelabel', 'somename').for_password()
+    password = TextInput("somelabel", "somename").for_password()
     actual = HelperHTMLParser.parse(str(password))
-    expected = HelperHTMLParser.parse(f'''
+    expected = HelperHTMLParser.parse(
+        f"""
         <div class="form-group row">
-            <label class="col-sm-2 col-form-label d-flex align-items-center"
+            <label class="col-sm-4 col-form-label"
                 for="{password.identifier}">
                 somelabel
             </label>
-            <div class="col-sm-10 d-flex align-items-center">
+            <div class="col-sm-8">
                 <input id="{password.identifier}"
                     name="somename"
                     type="password"
                     class="form-control"/>
             </div>
         </div>
-    ''')
+    """
+    )
     assert actual == expected
 
     # testing exception...
     with pytest.raises(AssertionError):
         str(
-            TextInput('somelabel', 'somename', 'somevalue').
-            with_multirows(5).for_email()
+            TextInput("somelabel", "somename", "somevalue")
+            .with_multirows(5)
+            .for_email()
         )
     with pytest.raises(AssertionError):
         str(
-            TextInput('somelabel', 'somename', 'somevalue').
-            with_multirows(5).for_password()
+            TextInput("somelabel", "somename", "somevalue")
+            .with_multirows(5)
+            .for_password()
         )
 
 
 @pytest.mark.form
 def test_text_area():
     def get_expected(element, disabled=False):
-        return f'''
+        return f"""
             <div class="form-group row">
-                <label class="col-sm-2 col-form-label d-flex
-                    align-items-center"
+                <label class="col-sm-4 col-form-label"
                     for="{element.identifier}">
                     somelabel
                 </label>
-                <div class="col-sm-10 d-flex align-items-center">
+                <div class="col-sm-8">
                     <textarea id="{element.identifier}"
                         name="somename"
                         class="form-control"
@@ -229,18 +419,22 @@ def test_text_area():
                         >somevalue</textarea>
                 </div>
             </div>
-        '''
+        """
 
     textarea = TextInput(
-        'somelabel', 'somename', 'somevalue', placeholder='someplaceholder'
+        "somelabel", "somename", "somevalue", placeholder="someplaceholder"
     ).with_multirows(5)
     actual = HelperHTMLParser.parse(str(textarea))
     expected = HelperHTMLParser.parse(get_expected(textarea))
     assert actual == expected
 
-    textarea = TextInput(
-        'somelabel', 'somename', 'somevalue', placeholder='someplaceholder'
-    ).with_multirows(5).as_disabled()
+    textarea = (
+        TextInput(
+            "somelabel", "somename", "somevalue", placeholder="someplaceholder"
+        )
+        .with_multirows(5)
+        .as_disabled()
+    )
     actual = HelperHTMLParser.parse(str(textarea))
     expected = HelperHTMLParser.parse(get_expected(textarea, disabled=True))
     assert actual == expected
@@ -249,16 +443,17 @@ def test_text_area():
 @pytest.mark.form
 def test_numeric_input():
     numeric = NumericInput(
-        'somelabel', 'somename', 'somevalue', placeholder='someplaceholder'
+        "somelabel", "somename", "somevalue", placeholder="someplaceholder"
     )
     actual = HelperHTMLParser.parse(str(numeric))
-    expected = HelperHTMLParser.parse(f'''
+    expected = HelperHTMLParser.parse(
+        f"""
         <div class="form-group row">
-            <label class="col-sm-2 col-form-label d-flex align-items-center"
+            <label class="col-sm-4 col-form-label"
                 for="{numeric.identifier}">
                 somelabel
             </label>
-            <div class="col-sm-10 d-flex align-items-center">
+            <div class="col-sm-8">
                 <input id="{numeric.identifier}"
                     name="somename"
                     value="somevalue"
@@ -267,26 +462,28 @@ def test_numeric_input():
                     placeholder="someplaceholder"/>
             </div>
         </div>
-    ''')
+    """
+    )
     assert actual == expected
 
 
 @pytest.mark.form
 def test_select_input():
-    option_0 = SelectInput.Option('Zero', 0, False)
-    option_1 = SelectInput.Option('One', 1, True)
+    option_0 = SelectInput.Option("Zero", 0, False)
+    option_1 = SelectInput.Option("One", 1, True)
     options = [option_0, option_1]
 
     # testing select-input...
-    select = SelectInput('somelabel', 'somename', 0, options)
+    select = SelectInput("somelabel", "somename", 0, options)
     actual = HelperHTMLParser.parse(str(select))
-    expected = HelperHTMLParser.parse(f'''
+    expected = HelperHTMLParser.parse(
+        f"""
         <div class="form-group row">
-            <label class="col-sm-2 col-form-label d-flex align-items-center"
+            <label class="col-sm-4 col-form-label"
                 for="{select.identifier}">
                 somelabel
             </label>
-            <div class="col-sm-10 d-flex align-items-center">
+            <div class="col-sm-8">
                 <select id="{select.identifier}"
                     name="somename"
                     class="form-control"
@@ -301,18 +498,20 @@ def test_select_input():
                 </select>
             </div>
         </div>
-    ''')
+    """
+    )
     assert actual == expected
 
-    select = SelectInput('somelabel', 'somename', 0, options).as_disabled()
+    select = SelectInput("somelabel", "somename", 0, options).as_disabled()
     actual = HelperHTMLParser.parse(str(select))
-    expected = HelperHTMLParser.parse(f'''
+    expected = HelperHTMLParser.parse(
+        f"""
         <div class="form-group row">
-            <label class="col-sm-2 col-form-label d-flex align-items-center"
+            <label class="col-sm-4 col-form-label"
                 for="{select.identifier}">
                 somelabel
             </label>
-            <div class="col-sm-10 d-flex align-items-center">
+            <div class="col-sm-8">
                 <select id="{select.identifier}"
                     name="somename"
                     class="form-control"
@@ -328,27 +527,29 @@ def test_select_input():
                 </select>
             </div>
         </div>
-    ''')
+    """
+    )
     assert actual == expected
 
 
 @pytest.mark.form
 def test_radio_input():
-    option_0 = SelectInput.Option('Zero', 0, False)
-    option_1 = SelectInput.Option('One', 1, True)
+    option_0 = SelectInput.Option("Zero", 0, False)
+    option_1 = SelectInput.Option("One", 1, True)
     options = [option_0, option_1]
 
     # testing select-input...
-    radio = SelectInput('somelabel', 'somename', 0, options).as_radio()
+    radio = SelectInput("somelabel", "somename", 0, options).as_radio()
     actual = HelperHTMLParser.parse(str(radio))
-    expected = HelperHTMLParser.parse(f'''
+    expected = HelperHTMLParser.parse(
+        f"""
         <div class="form-group row">
-            <label class="col-sm-2 col-form-label d-flex align-items-center"
+            <label class="col-sm-4 col-form-label"
                 for="{radio.identifier}">
                 somelabel
             </label>
-            <div class="col-sm-10 d-flex align-items-center">
-                <div class="form-check mr-3">
+            <div class="col-sm-8">
+                <div class="form-check me-3">
                     <input id="{option_0.identifier}"
                         name="somename"
                         value=0
@@ -356,12 +557,12 @@ def test_radio_input():
                         class="form-check-input"
                         autocomplete="off"
                         checked/>
-                    <label class="form-check-label mr-1"
+                    <label class="form-check-label me-1"
                         for="{option_0.identifier}">
                         Zero
                     </label>
                 </div>
-                <div class="form-check mr-3">
+                <div class="form-check me-3">
                     <input id="{option_1.identifier}"
                         name="somename"
                         value=1
@@ -369,42 +570,90 @@ def test_radio_input():
                         class="form-check-input"
                         autocomplete="off"
                         disabled/>
-                    <label class="form-check-label mr-1"
+                    <label class="form-check-label me-1"
                         for="{option_1.identifier}">
                         One
                     </label>
                 </div>
             </div>
         </div>
-    ''')
+    """
+    )
+    assert actual == expected
+
+
+@pytest.mark.form
+def test_json_input():
+    # testing json-input...
+    json = JsonInput("somelabel", "somename", {"hello": "test"})
+    actual = HelperHTMLParser.parse(str(json))
+    expected = HelperHTMLParser.parse(
+        f"""
+        <div class="form-group row">
+            <label class="col-sm-4 col-form-label"
+                for="{json.identifier}">
+                somelabel
+            </label>
+            <div class="col-sm-8">
+                <pre contenteditable="true" class="w-100" onkeyup="javascript:$('#{json.identifier}').val($(this).text())">
+                    <code class="language-json">...</code>
+                </pre>
+                <input id="{json.identifier}" name="somename" value="..." type="hidden"></input>
+            </div>
+        </div>
+    """
+    )
+    assert actual == expected
+
+    json = JsonInput("somelabel", "somename", {"hello": "test"}).as_disabled()
+    actual = HelperHTMLParser.parse(str(json))
+    expected = HelperHTMLParser.parse(
+        f"""
+        <div class="form-group row">
+            <label class="col-sm-4 col-form-label"
+                for="{json.identifier}">
+                somelabel
+            </label>
+            <div class="col-sm-8">
+                <pre contenteditable="false" class="w-100" onkeyup="javascript:$('#{json.identifier}').val($(this).text())">
+                    <code class="language-json">...</code>
+                </pre>
+                <input id="{json.identifier}" name="somename" value="..." type="hidden"></input>
+            </div>
+        </div>
+    """
+    )
     assert actual == expected
 
 
 @pytest.mark.form
 def test_hidden_input():
-    hidden = HiddenInput('somename', 'somevalue')
+    hidden = HiddenInput("somename", "somevalue")
     actual = HelperHTMLParser.parse(str(hidden))
-    expected = HelperHTMLParser.parse(f'''
+    expected = HelperHTMLParser.parse(
+        f"""
         <input id="{hidden.identifier}"
             name="somename"
             value="somevalue"
             type="hidden"/>
-    ''')
+    """
+    )
     assert actual == expected
 
 
 @pytest.mark.form
 def test_file_input():
-    regexp = r'/[\|/]/'
-    file = FileInput('somelabel', 'somename').add_classes('someclass')
+    regexp = r"/[\|/]/"
+    file = FileInput("somelabel", "somename").add_classes("someclass")
     actual = HelperHTMLParser.parse(str(file))
-    expected = HelperHTMLParser.parse(f'''
+    expected = HelperHTMLParser.parse(
+        f"""
         <div class="form-group someclass row">
-            <label class="col-sm-2 col-form-label d-flex align-items-center"
+            <label class="col-sm-4 col-form-label"
                 for="{file.identifier}">
                 somelabel
             </label>
-            <div class="col-sm-10 d-flex align-items-center">
+            <div class="col-sm-8">
                 <div class="input-group">
                     <span class="form-control input-group-append"></span>
                     <div class="input-group-append">
@@ -421,18 +670,20 @@ def test_file_input():
                 </div>
             </div>
         </div>
-    ''')
+    """
+    )
     assert actual == expected
 
-    file = FileInput('somelabel', 'somename').as_disabled()
+    file = FileInput("somelabel", "somename").as_disabled()
     actual = HelperHTMLParser.parse(str(file))
-    expected = HelperHTMLParser.parse(f'''
+    expected = HelperHTMLParser.parse(
+        f"""
         <div class="form-group row">
-            <label class="col-sm-2 col-form-label d-flex align-items-center"
+            <label class="col-sm-4 col-form-label"
                 for="{file.identifier}">
                 somelabel
             </label>
-            <div class="col-sm-10 d-flex align-items-center">
+            <div class="col-sm-8">
                 <div class="input-group">
                     <span class="form-control input-group-append"></span>
                     <div class="input-group-append">
@@ -450,5 +701,56 @@ def test_file_input():
                 </div>
             </div>
         </div>
-    ''')
+    """
+    )
+    assert actual == expected
+
+
+@pytest.mark.button
+def test_button_group():
+    # input group without tip...
+    input_group = InputGroup(
+        Text("@"), TextInput(None, "username", placeholder="type username")
+    )
+    actual = HelperHTMLParser.parse(str(input_group))
+    expected = HelperHTMLParser.parse(
+        """
+        <div id="..."
+            class="input-group"
+            role="group">
+            <span id="..." class="input-group-text">@</span>
+            <input id="..."
+                name="username"
+                type="text"
+                class="form-control"
+                placeholder="type username"
+                />
+        </div>
+    """
+    )
+    assert actual == expected
+
+    # input group with tip...
+    input_group = InputGroup(
+        Text("@"), TextInput(None, "username", placeholder="type username")
+    ).with_tip("Here is some tip")
+    actual = HelperHTMLParser.parse(str(input_group))
+    expected = HelperHTMLParser.parse(
+        """
+        <div id="..."
+            class="input-group"
+            role="group">
+            <span id="..." class="input-group-text">@</span>
+            <input id="..."
+                name="username"
+                type="text"
+                class="form-control"
+                placeholder="type username"
+                />
+        </div>
+        <small class="form-text text-secondary" style="font-size: 0.75em;">
+            Here is some tip
+        </small>
+    """
+    )
     assert actual == expected
